# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['flatql', 'flatql.parser', 'flatql.rewriter']

package_data = \
{'': ['*']}

install_requires = \
['ply>=3.11,<4.0', 'pyhocon>=0.3.59,<0.4.0', 'schema>=0.7.5,<0.8.0']

setup_kwargs = {
    'name': 'python-flatql',
    'version': '0.1.0',
    'description': 'Compile flat query language into SQL dialect for data analysis',
    'long_description': '# Flat-QL\n[![tests](https://github.com/Agile-Data/flat-ql/actions/workflows/tests.yml/badge.svg)](https://github.com/Agile-Data/flat-ql/actions/workflows/tests.yml)\n[![codecov](https://codecov.io/gh/Agile-Data/flat-ql/branch/main/graph/badge.svg?token=NQDHER8OF9)](https://codecov.io/gh/Agile-Data/flat-ql)\n\nFlat-QL is a query language for data analysis in SQL databases, it does not have `JOIN`, `GROUP BY`, `SUBQUERY` and more.\nIn Flat-QL, you don\'t to think of relational calculations and over-counting, which has only projections, filters, order by and limit.\nThe first, you need to define your data models and relationships using `hocon` format (you can also redefine it to whatever format you want), \nthen you just need write a SQL-like language (based on sales data structure) as follows.\n\n## Foreign Sums And Counts\n\nFlat-QL allows you to compute sums, counts correctly based on your join tree. This example has `Sales`, joining to `Product`, joining \nto `Store` and `Guider`. `Slaes` has a number of `Product`, `Store` and `Guider`. Code below computes sums, counts and averages at various places in the join tree.\nSee more: [sales schema definitions](https://github.com/Agile-Data/flat-ql/tree/main/tests/schemas/sales)\n![img.png](doc/1.png)\n\n```python\nimport os\nfrom flatql import parse_from_hocon_path, parse_flatql\nfrom flatql.rewriter import SqlRewriter\n\nsales_schema = parse_from_hocon_path(f"{os.path.dirname(__file__)}/schemas/sales")\nsql_rewriter = SqlRewriter(sales_schema)\n\n# As you can see, there are multi-entity(or tables) in the SQL-like language, but no joins, no group bys\nparse_flatql(\'SELECT Store.city AS "City", COUNT(Product.name) AS "Count Of Product", \'\n             \'SUM(Sales.sales) AS "Sales", AVG(Sales.quantityPurchased) AS "Average Sales" FROM sales \'\n             \'WHERE Store.city IN (\\\'New York\\\', \\\'Chicago\\\') ORDER BY "City" LIMIT 10 OFFSET 1\').rewrite(sql_rewriter)\nprint(sql_rewriter.to_sql())\n```\n\n```sql\n-- Compiled SQL\nSELECT "qu_0"."co_1" AS "City",\n       "qu_0"."co_3" AS "Count Of Product",\n       "qu_1"."co_7" AS "Sales",\n       "qu_1"."co_8" AS "Average Sales"\nFROM (SELECT "qu_2"."co_1" AS "co_1", SUM("qu_3"."co_2") AS "co_3"\n      FROM (SELECT "ta_0"."id" AS "co_0", "ta_1"."city" AS "co_1"\n            FROM "db"."sales" AS "ta_2"\n                     INNER JOIN "db"."products" AS "ta_0" ON "ta_0"."id" = "ta_2"."productId"\n                     INNER JOIN "db"."stores" AS "ta_1" ON "ta_1"."id" = "ta_2"."storeId"\n            WHERE "ta_1"."city" IN (\'New York\', \'Chicago\')\n            GROUP BY "co_0", "co_1") AS "qu_2"\n               LEFT JOIN (SELECT "ta_0"."id" AS "co_0", COUNT("ta_0"."name") AS "co_2"\n                          FROM "db"."products" AS "ta_0"\n                          GROUP BY "co_0") AS "qu_3" ON "qu_2"."co_0" = "qu_3"."co_0"\n      GROUP BY "co_1") AS "qu_0"\n         INNER JOIN (SELECT "qu_4"."co_1" AS "co_1", SUM("qu_5"."co_5") AS "co_7", AVG("qu_5"."co_6") AS "co_8"\n                     FROM (SELECT "ta_2"."id" AS "co_4", "ta_1"."city" AS "co_1"\n                           FROM "db"."sales" AS "ta_2"\n                                    INNER JOIN "db"."stores" AS "ta_1" ON "ta_1"."id" = "ta_2"."storeId"\n                           WHERE "ta_1"."city" IN (\'New York\', \'Chicago\')\n                           GROUP BY "co_4", "co_1") AS "qu_4"\n                              LEFT JOIN (SELECT "ta_2"."id"                     AS "co_4",\n                                                SUM("ta_2"."sales")             AS "co_5",\n                                                AVG("ta_2"."quantityPurchased") AS "co_6"\n                                         FROM "db"."sales" AS "ta_2"\n                                         GROUP BY "co_4") AS "qu_5" ON "qu_4"."co_4" = "qu_5"."co_4"\n                     GROUP BY "co_1") AS "qu_1" ON "qu_0"."co_1" = "qu_1"."co_1"\nORDER BY "City" ASC LIMIT 10 OFFSET 1\n```\n\nIn the compiled SQL above, the relations of multi-table is calculated by Flat-QL, `COUNT` and `SUM` aggregate functions are\nseparated in isolated sub-queries for preventing over-count(or duplicated data generated by Cartesian Product).\n\n## Goals\n\n* Support normal aggregate functions(sum, count, avg. etc)\n* Support Logical Expressions and Binary Expressions for customizing the metrics\n* Support Pivot and UnPivot for data analysis\n* Support Window Functions encapsulated as a function call in SQL-like language',
    'author': 'YeongHer.Wong',
    'author_email': 'github@agidata.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Agile-Data/flat-ql',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.0,<4.0.0',
}


setup(**setup_kwargs)
